package com.ca.common.util;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;

public class CheckLastLoginDate {

	public Hashtable<String, Date> readInput(String inputFile) throws Throwable{
		Hashtable<String, Date> hashtable = new Hashtable<String, Date>();
		
		FileReader fr = null;
		BufferedReader br = null;
		try {
			File file = new File(inputFile);
			if (!file.exists()) {
				throw new Throwable("Input file is not exist");
			}
			fr = new FileReader(inputFile);
			br = new BufferedReader(fr);
			
			String line = null;
			while((line = br.readLine()) != null) {
				if (!line.contains("uid=") || !line.contains("LastLoginTime:")) {
					continue;
				}
                getUserLastLoginDate(line, hashtable);
            }
			
			return hashtable;
		} catch (Exception ex) {
			ex.printStackTrace();
			throw ex;
		} finally {
			try {
				br.close();
				fr.close();
			} catch (IOException ioe) {
				ioe.printStackTrace();
				throw ioe;
			}
		}
	}
	
	private void getUserLastLoginDate(String loginData, Hashtable<String, Date> hashtable) throws Throwable{
		try {
			String uid = getUID(loginData);
			Date lastLoginDate = getLastLoginDate(loginData);
			hashtable.put(uid, lastLoginDate);
		} catch (Exception ex) {
			ex.printStackTrace();
			throw ex;
		}
	}
	
	private String getUID (String loginData) throws Throwable{
		try {
			String uid = loginData.substring(loginData.indexOf("uid=") + "uid=".length(), loginData.indexOf(','));
			return uid;
		} catch (Exception ex) {
			ex.printStackTrace();
			throw ex;
		}
	}
	
	private Date getLastLoginDate (String loginData) throws Throwable{
		try {
			String strLastLoginDate = loginData.substring(loginData.indexOf("LastLoginTime:") + "LastLoginTime:".length(), loginData.length());
			@SuppressWarnings("deprecation")
			Date lastLoginDate = new Date(Date.parse(strLastLoginDate));
			return lastLoginDate;
		} catch (Exception ex) {
			ex.printStackTrace();
			throw ex;
		}
	}
	
	
	
	public void writeOutput(String outputFile, Hashtable<String, Date> hashtable) throws Throwable{

		FileWriter fw = null;
		BufferedWriter bw = null;
		try {
			File file = new File(outputFile);
			if (file.exists()) {
				Date today = new Date();
				SimpleDateFormat transFormat = new SimpleDateFormat("yyyyMMddHHmmss");
				String to = transFormat.format(today);
				file.renameTo(new File(outputFile + "_" + to));
			}
			
			fw = new FileWriter(outputFile);
			bw = new BufferedWriter(fw);
			bw.write("action,uid \r\n");
			
			Enumeration<String> keys = hashtable.keys();
			while(keys.hasMoreElements()) {
				bw.write("delete,");
				bw.write(keys.nextElement() + "\r\n");
			}
		} catch (Exception ex) {
			ex.printStackTrace();
			throw ex;
		} finally {
			try {
				bw.close();
				fw.close();
			} catch (IOException ioe) {
				ioe.printStackTrace();
				throw ioe;
			}
		}
	}
	
	private void printUsage() {
		System.out.println("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
		System.out.println("+ Usage : ");
		System.out.println("+ For Windows : run.sh(run.cmd) [full path of input file] [full path of output file] [range of day]");
		System.out.println("+    run.cmd c:/tmp/logincheck/input/input.txt /tmp/logincheck/out/output.txt 365");
		System.out.println("+ For Linux : run.sh(run.cmd) -input=[full path of input file] -output=[full path of output file] -range=[range of day]");
		System.out.println("+    run.sh -input=/tmp/logincheck/input/input.txt -output=/tmp/logincheck/out/output.txt -range=365");
		System.out.println("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
		System.exit(0);
	}
	
	public static void main(String[] args) {
		
		CheckLastLoginDate checkLastLoginDate = new CheckLastLoginDate();
		
		int range = 365;
		String inputFile = null;
		String outputFile = null;
		if (args.length < 3) {
			checkLastLoginDate.printUsage();
		} else {
			for (String arg : args) {
				try {
				arg = arg.trim();
				if (arg.toLowerCase().startsWith("-input=")) {
					inputFile = arg.substring("-input=".length(), arg.length());
				}
				if (arg.toLowerCase().startsWith("-output")) {
					outputFile = arg.substring("-output=".length(), arg.length());
				}
				if (arg.toLowerCase().startsWith("-range")) {
					String strRange = arg.substring("-range=".length(), arg.length());
					try {
						range = Integer.parseInt(strRange);
					} catch (Exception ex) {
						checkLastLoginDate.printUsage();
					}
				}
				} catch (Exception ex) {
					checkLastLoginDate.printUsage();
				}
			}
		}
		
		try {
			System.out.println("++ Started to read the file !!");
			Hashtable<String, Date> hashtable = checkLastLoginDate.readInput(inputFile);
			Enumeration<String> keys = hashtable.keys();
			Date today = new Date();
			while(keys.hasMoreElements()) {
				String key = keys.nextElement();
				Date lastLoginDate = hashtable.get(key);
				long lastLoginTime = lastLoginDate.getTime();
				long todayTime = today.getTime();
				
				long diff = todayTime - (lastLoginTime + (range * 24 * 60 * 60 * 1000));
				if (diff < 0) {
					hashtable.remove(key);
				}
			}
			System.out.println("++ Completed to check the last login time for all user !!");
			
			
			checkLastLoginDate.writeOutput(outputFile, hashtable);
			
			System.out.println("++ Finished to create the output file !!");
			
		} catch (Throwable t) {
			System.out.println("++ Program got below exception : ");
			t.printStackTrace();
			System.exit(1);
		} finally {
			System.exit(0);
		}
	}
	
}
